
%    Sparse Coding Classification Code -- (c) 2009-2011 Simon Scholler,
%    Music Technology Group, UPF, Barcelona,
%    and Bernstein Center of Computational Neuroscience, Berlin
%
%    This code is provided as it is, with no guarantees.  Published reports of research
%    using this code (or a modified version, maintaining a significant portion of the original code)
%    should cite the following article:
%
%    Scholler, S., Purwins, H., Sparse Approximations for Drum Sound Classification,
%    IEEE Journal of Selected Topics in Signal Processing, Vol. 5, No. 5: pp. 933-940, 2011 
%
%    Comments and bug reports are welcome.  Email to
%    simon.scholler@googlemail.com.
%    I would also appreciate hearing about how you used this code,
%
%    You are free to use this code and
%    to modify, extend or distribute this code, as long
%    as this copyright notice is included whole and unchanged.
%


function atomfcn = spac(data,opt)
%
% Usage: out = spac(data,opt)
%
% IN:   data        -       four types of inputs are possible:
%                             (1) a matrix (columns are interpreted as input vectors)
%                             (2) a cell array of vectors
%       opt         -       a setting options structure (see get_def_opt() for details)
% OUT:  atomfcn       -       cell array of learned atomic functions
%

global DICTLEN PL THRESHI_FRAC

if nargin==1 && iscell(data) && length(data)==2
    opt = data{2};
    data = data{1};
end

THRESHI_FRAC = opt.threshi_frac;

if ismatrix(data)  % input signal are given as columns of a matrix
   sounds = cell(1,size(data,2));
   for n = 1:size(data,2)
       sounds{n} = data(:,n);
   end
elseif iscell(data)
   sounds = data;
else
    error('data input format unknown')
end
nSig = length(sounds); 
opt.nSig = nSig;


%% randomly initialize atomic functions & normalize them to l2-norm 1
atomiclen = DICTLEN;  % length of the atomic functions
atomfcn = cell(1,opt.nAF);
if opt.vary_AFsize
   if sum(opt.AFsizes(2,:))~=1 && sum(opt.AFsizes(2,:))~=opt.nAF
      error('opt.AFsizes(2,:) is not valid.')
   end
   bf_idx = 1;
   for n = 1:size(opt.AFsizes,2)
      len = round(opt.AFsizes(1,n)*atomiclen);  
      if sum(opt.AFsizes(2,:))==1        
         howmany = round((opt.AFsizes(2,n)*opt.nAF));
      else
         howmany = opt.AFsizes(2,n);
      end      
      for bf = 1:howmany          
          atomfcn{bf_idx} = randn(len,1);
          bf_idx = bf_idx+1;
      end
   end            
   opt.nAF = length(atomfcn);
else    
    for bf = 1:opt.nAF
       atomfcn{bf} = randn(atomiclen,1);
    end
end
if opt.vary_atomsize
    for bf = 1:length(atomfcn)
       if isempty(atomfcn{bf})
           atomfcn(bf) = [];
       else
           padlen = round((PL/10)*length(atomfcn{bf})/(10+2*PL/10));
           atomfcn{bf}([1:padlen end-padlen+1:end]) = zeros(padlen*2,1);
       end
    end
end
atomfcn = dict_normalize(atomfcn);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SC Optimization
opt.It = ceil(opt.nUpdates/nSig);
count = counter();

for k = 1:opt.It 
   snr_reached = zeros(nSig,1);
   %% LOOP OVER ALL SOUNDS
   for s = 1:nSig   
      if opt.It==1
          disp([int2str(s) '/' int2str(nSig)])
      end
      sound = sounds{s};
      if length(sound)<DICTLEN
          sound = [sound; zeros(DICTLEN-length(sound),1)]; % pad with zeros to reach required length
      end            
      book = mpd(sound,atomfcn,opt.mp);   % MP-decompose sound      
      snr_reached(s) = book.snr>=opt.mp.SNR;  
      
      %%% GRADIENT DESCENT UPDATE STEP %%%
      for i=1:size(book.X,2),       
          bi = book.idx(i);
          res_atomscope = book.residual(book.shift(i):book.shift(i)+length(atomfcn{bi})-1);
          atomfcn{bi} = atomfcn{bi} + opt.mu*book.w(i)*res_atomscope;
      end
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      if opt.normalizeAFs
          atomfcn = dict_normalize(atomfcn);
      end
      if opt.vary_atomsize
          ubi = unique(book.idx);
          atomfcn = update_atomsize(atomfcn,ubi);
      end
      count = count.oneup();
   end
end % iteration loop


