%    Sparse Coding Classification Code -- (c) 2009-2011 Simon Scholler,
%    Music Technology Group, UPF, Barcelona,
%    and Bernstein Center of Computational Neuroscience, Berlin
%
%    This code is provided as it is, with no guarantees.  Published reports of research
%    using this code (or a modified version, maintaining a significant portion of the original code)
%    should cite the following article:
%
%    Scholler, S., Purwins, H., Sparse Approximations for Drum Sound Classification,
%    IEEE Journal of Selected Topics in Signal Processing, Vol. 5, No. 5:pp. 933-940, 2011 
%
%    Comments and bug reports are welcome.  Email to
%    simon.scholler@googlemail.com.
%    I would also appreciate hearing about how you used this code,
%
%    You are free to use this code and
%    to modify, extend or distribute this code, as long
%    as this copyright notice is included whole and unchanged.

addpath(genpath(fileparts(mfilename('fullpath'))))
addpath('util')
clear l a D Dn

global DICTLEN PL
DICTLEN = 128;   % maximal length of the atomic functions (use powers of 2)
PL = 10;         % atomic function padding size for shrinkage and growth (in % of the total length)

opt = get_def_opt();
opt.nAF = 3;
opt.AFsizes = [0.0625, 0.125, 0.25,  0.5, 1; ...  % AF length (ratio of DICTLEN)
               0, 0, 1, 0, 0];                    % fraction or total number of AFs with this length
opt.nUpdates = 1000;
opt.mu = 0.05;
opt.mp.corrthresh = 0.2;
opt.threshi_frac = 8;

% Create training data
N = 100;  % number of data vectors
L = DICTLEN;  % length
D = zeros(L,N);
Dn = zeros(L,N);

% construct two elementary functions
l(1)=31; 
a{1}=(sin(0:pi/(l(1)-1):pi)).^3;
l(2)=13;
a{2}=-sin(0:2*pi/(l(2)-1):2*pi);

for n=1:N,
    % compose training data from 4 instances of elem. functions
    for i=1:7, % streched/shifted versions of 1st elementary function
        off=round((L-l(1))*rand+0.5); % random offsets
        D(off:off+l(1)-1,n)=rand*a{1}; % random amplitude
    end
    
    for i=1:5, % streched/shifted versions of 2nd elementary function
        off=round((L-l(2))*rand+0.5);
        D(off:off+l(2)-1,n)=rand*a{2};
    end
    % introduce noise 
    Dn(:,n)=D(:,n)+1/30*randn(DICTLEN,1);
    Dn(:,n) = Dn(:,n)/l2norm(Dn(:,n));
end

figure
for n = 1:2
   subplot(2 ,1,n)
   plot_defaults
   plot(a{n})
   plot_defaults
   title(sprintf('Elementary Function Used for Train. Data Construct. #%i',n))
   box off   
end

D = [zeros(50,N); D; zeros(50,N)];
Dn = [zeros(50,N); Dn; zeros(50,N)];  % zero pad training data vectors (left and right) in order to diminish border effects
Dn = normcols(Dn);

figure, 
plot_defaults
imagesc(Dn), colorbar, title('Training Data (in Columns)')
plot_defaults
xlabel('No. of Training Function');

% Run sparse coding optimization & Plot resulting atomic functions
sr = spac(Dn,opt);
figure;
for n = 1:opt.nAF
   subplot(opt.nAF ,1,n)
   plot_defaults
   plot(sr{n})
   title(sprintf('Learned Atomic Function #%i',n))
   plot_defaults
   box off   
end

% reconstruct original signal no.1 with learned atomic functions

book=mpd(Dn(:,1),sr,opt.mp);
figure
plot(D(:,1)/l2norm(D(:,1)),'m'); 
hold on;
plot(Dn(:,1),'b');
plot(book.sigmodel,'r')
plot_defaults
title('Signal Reconstruction with Learned Atomic Functions')
legend('Clean Norm. Signal','Noisy Signal','Reconstruction')



